##############################################################################
# 
# Zope Public License (ZPL) Version 0.9.7
# ---------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""File objects"""

__version__='$Revision: 1.2 $'[11:-2]

import string, regex, regsub, time
from Globals import Persistent
from Acquisition import Acquirer
from Globals import HTML
from ImageFile import ImageFile

sep=regex.compile('[\\/]')


class FileObject(Acquirer,Persistent):
    """ """
    icon='misc_/Confera/attachment.gif'

    _types={'application/octet-stream':     'Binary File',
            'application/x-gzip' :          'Compressed File',
            'application/x-compress':       'Compressed File',
            'application/x-zip-compressed': 'Compressed File',
            'audio/basic':     'Audio File',
            'audio/x-aiff':    'Audio File',
            'audio/x-wav' :    'Audio File',
            'text/html':       'HTML Document',
            'text/asp':        'Active Server Document',
            'text/plain':      'Text Document',
            'image/x-ms-bmp':  'Bitmap Image',
            'image/bmp' :      'Bitmap Image',
            'image/gif' :      'GIF Image',
            'image/jpeg':      'JPEG Image',
            'image/x-png':     'PNG Image',
            'image/x-tiff':    'TIFF Image',
            'video/x-msvideo': 'Video File',
            'video/mpeg':      'Video File',
            'video/quicktime': 'Video File',
          }

    def __init__(self,file=''):
        if file:
            self._file    =file.read()
            name=string.strip(regsub.split(file.filename,sep)[-1])
            name=string.split(name, ' ')
            name=string.join(name, '_')
            self._name=name
            self._ctype   =string.strip(file.headers['content-type'])
            self._created =time.time()
            self._modified=self._created

    def content_type(self):
        return self._ctype

    def file_name(self):
        return self._name

    def file_bytes(self):
        return len(self._file)

    def file_kbytes(self):
        return int(len(self._file)/1024)

    def file_type(self):
        try:    return self._types[self._ctype]
        except KeyError:
            f=self._name
            return '%s File' % string.upper(f[(string.rfind('.', f)+1):])

    def date_created(self):
        t=time.localtime(self._created)
        return '%d/%d/%d %02d:%02d' % (t[1],t[2],t[0],t[3],t[4])

    def date_modified(self):
        t=time.localtime(self._modified)
        return '%d/%d/%d %02d:%02d' % (t[1],t[2],t[0],t[3],t[4])

    def __len__(self):
        try:    return len(self._file)
        except: return 0 # _file doesnt exist

    def __repr__(self, RESPONSE={}):
        """ """
        RESPONSE['content-type']=self._ctype
        return self._file

    index_html=__repr__

    info=HTML("""
<HTML>
<HEAD>
<TITLE>File Info</TITLE>
</HEAD>
<BODY BGCOLOR="#FFFFFF" LINK="#000099" VLINK="#555555">
<H2>File Info</H2>
<P>
<TABLE CELLSPACING="2">
<TR>
  <TD ALIGN="LEFT" VALIGN="TOP">
  <STRONG>Name</STRONG>
  </TD>
  <TD ALIGN="LEFT" VALIGN="TOP">
  <!--#var file_name-->
  </TD>
</TR>
<TR>
  <TD ALIGN="LEFT" VALIGN="TOP">
  <STRONG>Type</STRONG>
  </TD>
  <TD ALIGN="LEFT" VALIGN="TOP">
  <!--#var file_type-->
  </TD>
</TR>
<TR>
  <TD ALIGN="LEFT" VALIGN="TOP">
  <STRONG>Size</STRONG>
  </TD>
  <TD ALIGN="LEFT" VALIGN="TOP">
  <!--#var file_kbytes-->KB (<!--#var file_bytes--> bytes)
  </TD>
</TR>
<TR>
  <TD ALIGN="LEFT" VALIGN="TOP">
  <STRONG>Created</STRONG>
  </TD>
  <TD ALIGN="LEFT" VALIGN="TOP">
  <!--#var date_created-->
  </TD>
</TR>
<TR>
  <TD ALIGN="LEFT" VALIGN="TOP">
  <STRONG>Modified</STRONG>
  </TD>
  <TD ALIGN="LEFT" VALIGN="TOP">
  <!--#var date_modified-->
  </TD>
</TR>
<TR>
  <TD ALIGN="LEFT" VALIGN="TOP">
  <STRONG>Content-Type</STRONG>
  </TD>
  <TD ALIGN="LEFT" VALIGN="TOP">
  <!--#var content_type-->
  </TD>
</TR>
</TABLE>
</BODY>
</HTML>""")
